/* wptKeylist.h - Keylist element
 *	Copyright (C) 2001-2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
#ifndef WPT_KEYLIST_H
#define WPT_KEYLIST_H

#include "wptFileManager.h"

/* Available sort-by modes. */
enum key_sort_t {
    KEY_SORT_USERID	= 0,
    KEY_SORT_KEYID	= 1,
    KEY_SORT_IS_SECRET	= 2,
    KEY_SORT_LEN	= 3,
    KEY_SORT_VALIDITY	= 5,
    KEY_SORT_OTRUST	= 6,
    KEY_SORT_CREATED	= 7,
    KEY_SORT_ALGO	= 8,

    /* special attributes for signatures */
    SIG_SORT_CLASS	= 30,
    SIG_SORT_EXPIRE	= 31
};

/* Valid keylist modes. */
enum keylist_mode_t {	
    KEYLIST_ALL         =  0,	
    KEYLIST_LIST        =  1,
    KEYLIST_ENCRYPT	=  2, /* obsolete */
    KEYLIST_ENCRYPT_MIN =  4,	
    KEYLIST_SIGN        =  8,	
    KEYLIST_FLAG_FILE   = 16,
    KEYLIST_FLAG_SHORT  = 32
};

/* All possible key flags. */
enum key_flag_t {
    KEYFLAG_NONE     = 0,
    KEYFLAG_EXPIRED  = 1,
    KEYFLAG_REVOKED  = 2,
    KEYFLAG_DISABLED = 4
};

/* Symbolic column IDs. */
enum km_col_t {
    KM_COL_UID	    = 0,
    KM_COL_KEYID    = 1,
    KM_COL_TYPE	    = 2,
    KM_COL_SIZE	    = 3,
    KM_COL_CIPHER   = 4,
    KM_COL_VALID    = 5,
    KM_COL_TRUST    = 6,
    KM_COL_CREAT    = 7,
    KM_COL_DESC	    = 8
};

/* Symbolic column IDs for sig lists */
enum siglist_col_t {
    SL_COL_UID = 0,
    SL_COL_VALID = 1,
    SL_COL_CLASS = 2,
    SL_COL_CREATE = 3,
    SL_COL_KEYID = 4,
    SL_COL_EXPIRE = 5,
    SL_COL_ALGO = 6
};

/* Icon for the key types. */
enum key_image_t {
    KEY_IMG_PUB = 0,
    KEY_IMG_PAIR= 1,
    KEY_IMG_SORT_DOWN = 2,
    KEY_IMG_SORT_UP
};

/*-- keylist.c --*/
const char* get_key_pubalgo2 (gpgme_pubkey_algo_t alg);
const char* get_key_pubalgo (gpgme_pubkey_algo_t alg);
const char* get_key_algo( gpgme_key_t key, int keyidx );
const char* get_key_created( long timestamp );
const char* get_key_expire_date( long timestamp );
const char* get_key_type( gpgme_key_t key );
const char* get_key_size( gpgme_key_t key, int keyidx );
const char* get_key_fpr( gpgme_key_t key );
const char* get_key_trust (gpgme_key_t key, int uididx, int listmode);
const char* get_key_trust2 (gpgme_key_t key, int val, int uididx, int listmode);
const char* get_key_trust_str (int val);
const char* get_keyid_from_fpr (const char *fpr);

char* get_key_desc (gpgme_key_t key);

gpgme_user_id_t get_nth_userid (gpgme_key_t key, int idx);
int             count_userids (gpgme_key_t key);
gpgme_subkey_t  get_nth_key (gpgme_key_t key, int idx);
int		count_subkeys (gpgme_key_t key);
gpgme_key_sig_t get_selfsig (gpgme_key_sig_t sigs, const char *keyid, int first);

listview_ctrl_t keylist_load( HWND ctrl, gpg_keycache_t pubkc, gpg_keycache_t seckc, 
			     int mode, int sortby );
int keylist_reload( listview_ctrl_t lv, gpg_keycache_t c, int mode, int sortby );
void keylist_delete( listview_ctrl_t lv );
int keylist_add_key (listview_ctrl_t lv, int mode, 
		     struct keycache_s *ctx, gpgme_key_t key);
void keylist_upd_key (listview_ctrl_t lv, int pos, 
		      struct keycache_s *ctx, gpgme_key_t key);
void keylist_upd_col (listview_ctrl_t lv, int col);
gpgme_key_t* keylist_get_recipients( listview_ctrl_t lv, 
				        int *r_force_trust, int *r_count );
gpgme_key_t* keylist_enum_recipients( listview_ctrl_t lv,  int listype, int *r_count );
int keylist_sort( listview_ctrl_t lv, int sortby );

/*-- wptImportList.cpp --*/
int  implist_build( listview_ctrl_t *lv, HWND ctrl );
int  implist_load( listview_ctrl_t lv, const char *file, 
		   int *r_revcerts, int *r_seckeys );
void implist_delete( listview_ctrl_t lv );

/*-- wptSigList.cpp --*/
listview_ctrl_t siglist_load (HWND ctrl, const char *keyid);
void siglist_delete (listview_ctrl_t lv);
void siglist_sort (listview_ctrl_t sigl, int sortby);

/*-- wptVerifyList.cpp --*/
/* Verify list control context. */
struct verlist_ctrl_s {
    listview_ctrl_t lv;	    /* list view control. */
    HWND infctl;	    /* handle to the optional sig info control. */
};
typedef struct verlist_ctrl_s *verlist_ctrl_t;

void verlist_build (verlist_ctrl_t *vlv, HWND ctrl, int fm_mode);
void verlist_delete (verlist_ctrl_t vlv);
void verlist_alloc (verlist_ctrl_t vlv, int nentries );
void verlist_set_info_control (verlist_ctrl_t vlv, HWND infctl);
int verlist_add_sig (verlist_ctrl_t vlv, gpgme_signature_t sig);
int verlist_add_sig_log (verlist_ctrl_t vlv, file_sig_ctx_t log);

/*-- seclist --*/
struct keylist_s {
    struct keylist_s * next;
    gpgme_key_t key;
};
typedef struct keylist_s *keylist_t;

void seclist_destroy (keylist_t * list);
void seclist_init (HWND dlg, int ctlid, int flags, keylist_t * ret_list);
int  seclist_select_key (HWND dlg, int ctlid, gpgme_key_t * ret_key);

#endif /* WPT_KEYLIST_H */
